/*
 * Copyright © Original Authors
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Additional copyright and licensing notices may apply for content that was
 * included from other projects. For more information, see ATTRIBUTION.md.
 */

package grondag.xm;

import org.jetbrains.annotations.ApiStatus.Internal;
import net.fabricmc.api.ClientModInitializer;
import net.fabricmc.fabric.api.resource.ResourceManagerHelper;
import net.minecraft.class_1921;
import net.minecraft.class_2338;
import net.minecraft.class_243;
import net.minecraft.class_2680;
import net.minecraft.class_3264;
import net.minecraft.class_4696;
import net.minecraft.class_638;
import io.vram.frex.api.model.provider.ModelProviderRegistry;
import io.vram.frex.api.renderloop.BlockOutlineListener;
import io.vram.frex.api.renderloop.RenderReloadListener;
import io.vram.frex.api.rendertype.BlockPresets;

import grondag.xm.api.block.XmBlockState;
import grondag.xm.api.modelstate.ModelState;
import grondag.xm.collision.CollisionDispatcherImpl;
import grondag.xm.dispatch.XmDispatcher;
import grondag.xm.dispatch.XmVariantProvider;
import grondag.xm.mesh.helper.PolyTransformImpl;
import grondag.xm.modelstate.AbstractPrimitiveModelState;
import grondag.xm.network.Packets;
import grondag.xm.paint.XmPaintRegistryImpl;
import grondag.xm.primitive.ModelPrimitiveRegistryImpl;
import grondag.xm.render.OutlineRenderer;
import grondag.xm.texture.XmTexturesImpl;

@Internal
public class XmClient implements ClientModInitializer {
	@Override
	public void onInitializeClient() {
		XmTexturesImpl.init();

		ModelProviderRegistry.registerVariantProvider(r -> new XmVariantProvider());
		RenderReloadListener.register(XmClient::invalidate);
		Packets.initializeClient();
		AbstractPrimitiveModelState.useClientHandler();
		ResourceManagerHelper.get(class_3264.field_14188).registerReloadListener(XmPaintRegistryImpl.INSTANCE);

		SidedHelper.RENDER_LAYER_REMAPPER = (b, s) -> BlockPresets.mapBlocks(class_4696.method_23679(s), b);
		SidedHelper.RENDER_LAYER_REMAPS.forEach(SidedHelper.RENDER_LAYER_REMAPPER);
		SidedHelper.RENDER_LAYER_REMAPS.clear();

		BlockOutlineListener.register((ctx, btx) -> {
			final class_638 world = ctx.world();
			final class_2338 blockPos = btx.blockPos();
			final class_2680 blockState = btx.blockState();
			final ModelState modelState = XmBlockState.modelState(blockState, world, blockPos, true);

			if (modelState != null && !XmConfig.debugCollisionBoxes) {
				final class_243 cameraPos = ctx.camera().method_19326();

				OutlineRenderer.drawModelOutline(
					ctx.poseStack(),
					ctx.consumers().getBuffer(class_1921.method_23594()),
					modelState,
					blockPos.method_10263() - cameraPos.field_1352,
					blockPos.method_10264() - cameraPos.field_1351,
					blockPos.method_10260() - cameraPos.field_1350,
					0.0F, 0.0F, 0.0F, 0.4f
				);

				return false;
			}

			return true;
		});
	}

	public static void invalidate() {
		PolyTransformImpl.invalidateCache();
		XmDispatcher.INSTANCE.clear();
		CollisionDispatcherImpl.clear();
		ModelPrimitiveRegistryImpl.INSTANCE.invalidateCache();
	}
}
