/*
 * Copyright © Original Authors
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Additional copyright and licensing notices may apply for content that was
 * included from other projects. For more information, see ATTRIBUTION.md.
 */

package grondag.xm.api.block;

import java.util.function.BiFunction;
import java.util.function.Function;
import net.minecraft.class_1747;
import net.minecraft.class_1799;
import net.minecraft.class_1937;
import net.minecraft.class_2248;
import net.minecraft.class_2680;
import org.jetbrains.annotations.ApiStatus.Experimental;
import grondag.xm.api.modelstate.ModelStateFunction;
import grondag.xm.api.modelstate.MutableModelState;
import grondag.xm.dispatch.XmRegistryImpl;

@Experimental
public class XmBlockRegistry {
	private XmBlockRegistry() { }

	/**
	 * @deprecated Use version that accepts world for item state
	 */
	@Deprecated
	public static void addBlockStates(
			class_2248 block,
			Function<class_2680, ? extends ModelStateFunction<?>> modelFunctionMap,
			Function<class_1799, MutableModelState> itemModelFunction
	) {
		XmRegistryImpl.register(block, modelFunctionMap, (s, w) -> itemModelFunction.apply(s));
	}

	public static void addBlockStates(
			class_2248 block,
			Function<class_2680, ? extends ModelStateFunction<?>> modelFunctionMap,
					BiFunction<class_1799, class_1937, MutableModelState> itemModelFunction
	) {
		XmRegistryImpl.register(block, modelFunctionMap, itemModelFunction);
	}

	public static void addBlockStates(
			class_2248 block,
			Function<class_2680, ? extends ModelStateFunction<?>> modelFunctionMap
	) {
		XmRegistryImpl.register(block, modelFunctionMap, DEFAULT_ITEM_MODEL_FUNCTION_V2);
	}

	public static <F extends ModelStateFunction<?>> void addBlock(class_2248 block, F modelFunction) {
		addBlockStates(block, (class_2680 bs) -> modelFunction);
	}

	/**
	 * @deprecated Use version that accepts world for item state
	 */
	@Deprecated
	public static <F extends ModelStateFunction<?>> void addBlock(class_2248 block, F blockModelFunction, Function<class_1799, MutableModelState> itemModelFunction) {
		addBlockStates(block, (class_2680 bs) -> blockModelFunction, itemModelFunction);
	}

	public static <F extends ModelStateFunction<?>> void addBlock(class_2248 block, F blockModelFunction, BiFunction<class_1799, class_1937, MutableModelState> itemModelFunction) {
		addBlockStates(block, (class_2680 bs) -> blockModelFunction, itemModelFunction);
	}

	/**
	 * Use {@link #DEFAULT_ITEM_MODEL_FUNCTION_V2}.
	 */
	@Deprecated
	public static final Function<class_1799, MutableModelState> DEFAULT_ITEM_MODEL_FUNCTION = s -> {
		if (s.method_7909() instanceof class_1747) {
			final class_1747 item = (class_1747) s.method_7909();
			final XmBlockState xmState = XmBlockState.get(item);

			if (xmState != null) {
				return xmState.defaultModelState();
			}
		}

		return null;
	};

	public static final BiFunction<class_1799, class_1937, MutableModelState> DEFAULT_ITEM_MODEL_FUNCTION_V2 = (s, w) -> {
		if (s.method_7909() instanceof class_1747) {
			final class_1747 item = (class_1747) s.method_7909();
			final XmBlockState xmState = XmBlockState.get(item);

			if (xmState != null) {
				return xmState.defaultModelState();
			}
		}

		return null;
	};
}
