/*
 * Copyright © Original Authors
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Additional copyright and licensing notices may apply for content that was
 * included from other projects. For more information, see ATTRIBUTION.md.
 */

package grondag.xm.api.block;

import org.jetbrains.annotations.ApiStatus.Experimental;
import org.jetbrains.annotations.Nullable;
import grondag.xm.api.modelstate.MutableModelState;
import grondag.xm.dispatch.XmBlockStateAccess;
import net.minecraft.class_1747;
import net.minecraft.class_1922;
import net.minecraft.class_2248;
import net.minecraft.class_2338;
import net.minecraft.class_2680;

@Experimental
public interface XmBlockState {
	/**
	 * Minecraft block state associated with this Exotic Matter block state.
	 * Association is always 1:1.
	 */
	class_2680 blockState();

	/**
	 * If last parameter is false, does not perform a refresh from world for
	 * world-dependent state attributes. Use this option to prevent infinite
	 * recursion when need to reference some static state ) information in order to
	 * determine dynamic world state. Block tests are main use case for false.
	 */
	<T extends MutableModelState> T modelState(@Nullable class_1922 world, @Nullable class_2338 pos, boolean refreshFromWorld);

	default <T extends MutableModelState> T defaultModelState() {
		return modelState(null, null, false);
	}

	static @Nullable XmBlockState get(class_2680 fromState) {
		return ((XmBlockStateAccess) fromState).xm_toXmBlockState();
	}

	static @Nullable XmBlockState get(class_1747 fromItem) {
		return get(fromItem.method_7711());
	}

	static @Nullable XmBlockState get(class_2248 fromBlock) {
		return get(fromBlock.method_9564());
	}

	@SuppressWarnings("unchecked")
	static @Nullable <T extends MutableModelState> T modelState(class_2680 fromState, @Nullable class_1922 blockView, @Nullable class_2338 pos, boolean refresh) {
		final XmBlockState xmState = get(fromState);
		return xmState == null ? null : (T) xmState.modelState(blockView, pos, refresh);
	}
}
