/*
 * Copyright © Original Authors
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Additional copyright and licensing notices may apply for content that was
 * included from other projects. For more information, see ATTRIBUTION.md.
 */

package grondag.xm.dispatch;

import java.util.Collection;
import java.util.Collections;
import java.util.List;
import java.util.Random;
import java.util.Set;
import java.util.function.Function;

import org.jetbrains.annotations.ApiStatus.Internal;
import org.jetbrains.annotations.Nullable;
import var;
import com.mojang.datafixers.util.Pair;
import net.fabricmc.api.EnvType;
import net.fabricmc.api.Environment;
import net.minecraft.class_1058;
import net.minecraft.class_1087;
import net.minecraft.class_1088;
import net.minecraft.class_1100;
import net.minecraft.class_2338;
import net.minecraft.class_2350;
import net.minecraft.class_2680;
import net.minecraft.class_2960;
import net.minecraft.class_3665;
import net.minecraft.class_4730;
import net.minecraft.class_638;
import net.minecraft.class_777;
import io.vram.frex.api.buffer.QuadSink;

import grondag.xm.api.block.XmBlockState;
import grondag.xm.api.item.XmItem;
import grondag.xm.api.modelstate.MutableModelState;

@Internal
@Environment(EnvType.CLIENT)
public class XmModelProxy extends AbstractXmModel implements class_1100 {
	private XmModelProxy() { }

	public static final XmModelProxy INSTANCE = new XmModelProxy();

	@Override
	public List<class_777> method_4707(class_2680 state, class_2350 face, Random rand) {
		final XmBlockState xmState = XmBlockState.get(state);
		return xmState == null ? Collections.emptyList() : XmDispatcher.INSTANCE.get(xmState.defaultModelState()).bakedQuads(state, face, rand);
	}

	@Override
	public void renderAsBlock(BlockInputContext input, QuadSink output) {
		final MutableModelState modelState = XmBlockState.modelState(input.blockState(), input.blockView(), input.pos(), true);

		if (modelState != null) {
			XmDispatcher.INSTANCE.get(modelState).renderAsBlock(input, output);
			modelState.release();
		}
	}

	@Override
	public void renderAsItem(ItemInputContext input, QuadSink output) {
		final MutableModelState modelState = XmItem.modelState(input.itemStack());

		if (modelState != null) {
			XmDispatcher.INSTANCE.get(modelState).renderAsItem(input, output);
			modelState.release();
		}
	}

	@Override
	public Collection<class_2960> method_4755() {
		return Collections.emptyList();
	}

	@Override
	public Collection<class_4730> method_4754(Function<class_2960, class_1100> var1, Set<Pair<String, String>> var2) {
		return Collections.emptyList();
	}

	@Override
	public class_1087 method_4753(class_1088 var1, Function<class_4730, class_1058> var2, class_3665 var3, class_2960 modelId) {
		return this;
	}

	@Override
	public void onNewTerrainParticle(@Nullable class_638 clientLevel, class_2680 blockState, @Nullable class_2338 blockPos, TerrainParticleDelegate delegate) {
		final MutableModelState lookupState = XmBlockState.modelState(blockState, clientLevel, blockPos, false);

		if (lookupState != null) {
			final var renderState = XmDispatcher.INSTANCE.get(lookupState);
			lookupState.release();
			delegate.setModelParticleSprite(renderState.particleSprite());
			delegate.setModelParticleColor(renderState.particleColorARBG());
		}
	}
}
