/*
 * Copyright © Original Authors
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Additional copyright and licensing notices may apply for content that was
 * included from other projects. For more information, see ATTRIBUTION.md.
 */

package grondag.xm.network;

import io.netty.buffer.Unpooled;
import org.jetbrains.annotations.ApiStatus.Internal;
import net.minecraft.class_2338;
import net.minecraft.class_2540;
import net.minecraft.class_2596;
import net.minecraft.class_2960;
import net.minecraft.class_3222;
import net.minecraft.class_3244;
import net.minecraft.server.MinecraftServer;
import net.fabricmc.fabric.api.networking.v1.PacketSender;
import net.fabricmc.fabric.api.networking.v1.ServerPlayNetworking;

import grondag.fermion.position.IntegerBox;
import grondag.xm.Xm;
import grondag.xm.XmConfig;
import grondag.xm.virtual.ExcavationRenderEntry;
import grondag.xm.virtual.ExcavationRenderManager;
import grondag.xm.virtual.ExcavationRenderer;

/**
 * Adds, changes or removes an excavation render entry in current client list.
 * Sent when a new excavation is added, the bounds change, or excavation is
 * removed.
 */
@Internal
public abstract class S2C_ExcavationRenderUpdate {
	private S2C_ExcavationRenderUpdate() {
	}

	public static final class_2960 ID = new class_2960(Xm.MODID, "exru");

	/**
	 * Use this for new and changed.
	 */
	public static class_2596<?> toPacket(ExcavationRenderEntry entry) {
		final IntegerBox aabb = entry.aabb();
		final class_2338[] positions = entry.renderPositions();

		if (XmConfig.logExcavationRenderTracking) {
			Xm.LOG.info("id %d New update packet position count = %d, aabb=%s", entry.id, positions == null ? 0 : positions.length,
					aabb == null ? "null" : aabb.toString());
		}

		return toPacket(entry.id, aabb, entry.task.isExchange(), positions);
	}

	/**
	 * Use this for deleted.
	 */
	public static class_2596<?> toPacket(int deletedId) {
		return toPacket(deletedId, null, false, null);
	}

	private static class_2596<?> toPacket(int id, IntegerBox aabb, boolean isExchange, class_2338[] positions) {
		final class_2540 pBuff = new class_2540(Unpooled.buffer());

		if (XmConfig.logExcavationRenderTracking) {
			Xm.LOG.info("id %d Update toBytes position count = %d", id, positions == null ? 0 : positions.length);
		}

		pBuff.writeInt(id);
		// deletion flag
		pBuff.writeBoolean(aabb == null);

		if (aabb != null) {
			pBuff.writeLong(aabb.minPos().method_10063());
			pBuff.writeLong(aabb.maxPos().method_10063());
			pBuff.writeBoolean(isExchange);

			pBuff.writeInt(positions == null ? 0 : positions.length);

			if (positions != null) {
				for (final class_2338 pos : positions) {
					pBuff.writeLong(pos.method_10063());
				}
			}
		}

		return ServerPlayNetworking.createS2CPacket(ID, pBuff);
	}

	public static void accept(MinecraftServer server, class_3222 player, class_3244 handler, class_2540 buf, PacketSender responseSender) {
		// FIX: thread safety

		final int id = buf.readInt();
		final IntegerBox aabb;
		final class_2338[] positions;
		final boolean isExchange;

		if (buf.readBoolean()) {
			// deletion
			aabb = null;
			positions = null;
			isExchange = false;
		} else {
			final class_2338 minPos = class_2338.method_10092(buf.readLong());
			final class_2338 maxPos = class_2338.method_10092(buf.readLong());
			aabb = new IntegerBox(minPos, maxPos);
			isExchange = buf.readBoolean();

			final int posCount = buf.readInt();

			if (posCount == 0) {
				positions = null;
			} else {
				positions = new class_2338[posCount];

				for (int i = 0; i < posCount; i++) {
					positions[i] = class_2338.method_10092(buf.readLong());
				}
			}
		}

		if (XmConfig.logExcavationRenderTracking) {
			Xm.LOG.info("id %d Update fromBytes position count = %d", id, positions == null ? 0 : positions.length);
		}

		if (XmConfig.logExcavationRenderTracking) {
			Xm.LOG.info("id %d Update handler position count = %d, aabb=%s", id, positions == null ? 0 : positions.length,
					aabb == null ? "null" : aabb.toString());
		}

		if (aabb == null) {
			ExcavationRenderManager.remove(id);
		} else {
			ExcavationRenderManager.addOrUpdate(new ExcavationRenderer(id, aabb.toAABB(), isExchange, positions));
		}
	}
}
