/*
 * Copyright © Original Authors
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Additional copyright and licensing notices may apply for content that was
 * included from other projects. For more information, see ATTRIBUTION.md.
 */

package grondag.xm.terrain;

import org.jetbrains.annotations.ApiStatus.Internal;
import grondag.xm.api.block.XmBlockState;
import grondag.xm.api.modelstate.ModelState;
import grondag.xm.api.modelstate.MutableModelState;
import grondag.xm.api.terrain.TerrainModelState;
import net.minecraft.class_1657;
import net.minecraft.class_1937;
import net.minecraft.class_2248;
import net.minecraft.class_2338;
import net.minecraft.class_2680;

@Internal
public class TerrainDynamicBlock extends TerrainBlock {
	public TerrainDynamicBlock(class_2251 blockSettings, ModelState defaultModelState, boolean isFiller) {
		super(blockSettings, adjustShape(defaultModelState, isFiller));
	}

	// PERF: sucks
	private static TerrainModelState.Mutable adjustShape(ModelState stateIn, boolean isFiller) {
		final TerrainModelState.Mutable result = isFiller ? TerrainSurface.FILLER.newState() : TerrainSurface.HEIGHT.newState();
		result.copyFrom(stateIn);
		result.setStatic(false);
		return result;
	}

	/**
	 * Convert this block to a static version of itself if a static version was
	 * given.
	 */
	public void makeStatic(class_2680 state, class_1937 world, class_2338 pos) {
		final TerrainStaticBlock staticVersion = TerrainBlockRegistry.TERRAIN_STATE_REGISTRY.getStaticBlock(this);

		if (staticVersion == null || state.method_26204() != this) {
			return;
		}

		final MutableModelState myModelState = XmBlockState.modelState(state, world, pos, true);
		myModelState.setStatic(true);
		// TODO: transfer heat block state?
		world.method_8652(pos, staticVersion.method_9564().method_11657(TerrainBlock.TERRAIN_TYPE, state.method_11654(TerrainBlock.TERRAIN_TYPE)), 7);
		staticVersion.setModelState(world, pos, myModelState);
	}

	// TODO: restore or remove
	//    @Override
	//    public int quantityDropped(ExtendedBlockView world, BlockPos pos, BlockState state) {
	//        double volume = 0;
	//        ISuperModelState modelState = SuperBlockWorldAccess.access(world).computeModelState(this, state, pos, true);
	//        for (BoundingBox box : modelState.getShape().meshFactory().collisionHandler().getCollisionBoxes(modelState)) {
	//            volume += Useful.volumeAABB(box);
	//        }
	//
	//        return (int) Math.min(9, volume * 9);
	//    }

	// TODO: restore or remove
	//    @Override
	//    public boolean isReplaceable(ExtendedBlockView worldIn, BlockPos pos) {
	//        return SuperBlockWorldAccess.access(worldIn).terrainState(pos).isEmpty();
	//    }

	// TODO: restore or remove
	//    @Override
	//    public boolean isAir(BlockState state, ExtendedBlockView world, BlockPos pos) {
	//        return SuperBlockWorldAccess.access(world).terrainState(pos).isEmpty();
	//    }

	@Override
	public void method_9576(class_1937 world, class_2338 pos, class_2680 state, class_1657 player) {
		TerrainDynamicBlock.freezeNeighbors(world, pos, state);
		super.method_9576(world, pos, state, player);
	}

	/**
	 * Looks for nearby dynamic blocks that might depend on this block for height
	 * state and converts them to static blocks if possible.
	 */
	public static void freezeNeighbors(class_1937 worldIn, class_2338 pos, class_2680 state) {
		// only height blocks affect neighbors
		if (!TerrainBlockHelper.isFlowHeight(state)) {
			return;
		}

		class_2680 targetState;
		class_2248 targetBlock;

		for (int x = -2; x <= 2; x++) {
			for (int z = -2; z <= 2; z++) {
				for (int y = -4; y <= 4; y++) {
					//                    if(!(x == 0 && y == 0 && z == 0))
					{
						final class_2338 targetPos = pos.method_10069(x, y, z);
						targetState = worldIn.method_8320(targetPos);
						targetBlock = targetState.method_26204();

						if (targetBlock instanceof TerrainDynamicBlock) {
							((TerrainDynamicBlock) targetBlock).makeStatic(targetState, worldIn, targetPos);
						}
					}
				}
			}
		}
	}
}
