/*
 * Copyright © Original Authors
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Additional copyright and licensing notices may apply for content that was
 * included from other projects. For more information, see ATTRIBUTION.md.
 */

package grondag.xm.texture;

import java.util.function.Consumer;
import net.minecraft.class_1058;
import net.minecraft.class_2960;
import org.jetbrains.annotations.ApiStatus.Internal;
import grondag.xm.api.modelstate.ModelStateFlags;
import grondag.xm.api.texture.TextureSet;
import grondag.xm.api.texture.TextureSetBuilder;

@Internal
public class TextureSetImpl extends AbstractTextureSet implements TextureSet {
	public static TextureSetBuilder builder() {
		return new TextureSetBuilderImpl();
	}

	public static TextureSetBuilder builder(TextureSet template) {
		final TextureSetBuilderImpl result = new TextureSetBuilderImpl();
		result.copyFrom((AbstractTextureSet) template);
		return result;
	}

	public final class_2960 id;
	public final int versionMask;
	public final int stateFlags;
	public final String baseTextureName;
	private boolean used = false;

	TextureSetImpl(class_2960 id, AbstractTextureSet template) {
		this.id = id;
		baseTextureName = template.rawBaseTextureName;
		copyFrom(template);
		versionMask = Math.max(0, template.versionCount - 1);
		layoutMap = template.layoutMap;

		int flags = template.scale.modelStateFlag | template.layoutMap.layout.modelStateFlag;

		// textures with randomization options also require position information
		if (template.transform.hasRandom) {
			flags |= ModelStateFlags.POSITION;
		}

		if (template.versionCount > 1) {
			flags |= ModelStateFlags.POSITION;
		}

		stateFlags = flags;

		TextureSetRegistryImpl.INSTANCE.add(this);
	}

	@Override
	public class_2960 id() {
		return id;
	}

	@Override
	public int index() {
		return TextureSetRegistryImpl.INSTANCE.indexOf(this);
	}

	@Override
	public int stateFlags() {
		return stateFlags;
	}

	@Override
	public void prestitch(Consumer<class_2960> stitcher) {
		layoutMap.prestitch(this, stitcher);
	}

	@Override
	public String sampleTextureName() {
		return layoutMap.sampleTextureName(this);
	}

	private class_1058 sampleSprite;

	@Override
	public class_1058 sampleSprite() {
		class_1058 result = sampleSprite;

		if (result == null) {
			result = TextureSetHelper.blockAtas().method_4608(new class_2960(sampleTextureName()));
			sampleSprite = result;
		}

		return result;
	}

	@Override
	public String textureName(int version) {
		return layoutMap.buildTextureName(this, version & versionMask, 0);
	}

	@Override
	public String textureName(int version, int index) {
		return layoutMap.buildTextureName(this, version & versionMask, index);
	}

	@Override
	public int versionMask() {
		return versionMask;
	}

	@Override
	public String baseTextureName() {
		return baseTextureName;
	}

	@Override
	public void use() {
		used = true;
	}

	@Override
	public boolean used() {
		return used;
	}
}
